import {html, TemplateResult} from 'lit';
import {customElement, property} from 'lit/decorators.js';
import {repeat} from 'lit/directives/repeat.js';
import BaseFacetElement from '../BaseFacetElement';

/**
 * A facet checkbox element.
 */
@customElement('facet-checkbox')
export class FacetCheckbox extends BaseFacetElement {
  /**
   * Use radio inputs instead of checkboxes. Also ensures only one option can be chosen at a time.
   */
  @property({type: Boolean})
  useRadios = false;

  override connectedCallback() {
    super.connectedCallback();

    this.useRadios = this._configOrAttribute(
      this.facet?.settings.widget.useRadios,
      this.useRadios
    );
  }

  override applyFacet(
    key: string,
    value: string,
    clearApplied: boolean = false
  ) {
    //@TODO better support the `Ensure that only one result can be displayed` from facet config.
    //@TODO should clicking a radio unselect a selected value? Currently this code does not allow that.
    if (this.useRadios) {
      this.context?.query?.delete(`f:${key}`);
    }
    super.applyFacet(key, value, clearApplied);
  }

  /**
   * Returns the html for the facet's options.
   */
  _getOptions(): TemplateResult {
    const facet = this._getFacetData();

    return html`
      <ul>
        ${repeat(
          facet.results,
          (value) => value.key,
          (value) => {
            const id = this.genUniqId(
              `${facet.key}-${this.safeIdentifier(value.key)}`
            );
            return html`
              <li
                class="facet ${this.useRadios
                  ? 'radio'
                  : 'checkbox'} ${value.active ? 'selected' : ''}"
              >
                <input
                  type="${this.useRadios ? 'radio' : 'checkbox'}"
                  id=${id}
                  name="${value.key}"
                  value="${value.key}"
                  .checked=${value.active ?? false}
                  ?checked=${value.active ?? false}
                  @click=${() => this.applyFacet(this.urlAlias, value.key)}
                />
                <label for="${id}"
                  >${value.label}${this.showCount
                    ? ` (${value.count})`
                    : null}</label
                >
              </li>
            `;
          }
        )}
      </ul>
    `;
  }

  override render() {
    super.render();

    if (!this.shouldRender()) {
      return null;
    }

    if (this.collapsible) {
      return this.wrapCollapsible(
        this._getCollapsibleLabelElement(),
        html`${this._getOptions()} ${this._getResetElement()}`
      );
    }

    return html`
      ${this.showLabel ? this._getLabelElement() : null} ${this._getOptions()}
      ${this._getResetElement()}
    `;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'facet-checkbox': FacetCheckbox;
  }
}
