import {customElement, property} from 'lit/decorators.js';
import {BaseSearchElement} from '../BaseSearchElement';
import {html} from 'lit/static-html.js';
import {TemplateResult} from 'lit';

/**
 * A simple search pager that can display the current page, previous and next pages, previous and next buttons, first and last page buttons.
 */
@customElement('search-simple-pager')
export class SearchSimplePager extends BaseSearchElement {
  /**
   * The text to display for the previous button.
   */
  @property()
  prevLabel = '<';

  /**
   * The text to display for the next button.
   */
  @property()
  nextLabel = '>';

  /**
   * If the next and previous buttons should be shown.
   */
  @property({type: Boolean})
  showNextPrev = false;

  /**
   * If the first and last buttons should be shown
   */
  @property({type: Boolean})
  showFirstLast = false;

  /**
   * The text to display for the first page button.
   */
  @property()
  firstLabel = 'First page';

  /**
   * The text to display for the last page button.
   */
  @property()
  lastLabel = 'Last page';

  /**
   * Number of pages to display on either side of the current page.
   */
  @property({type: Number})
  pagesToDisplay: number = 0;

  /**
   * Update the current query with a new page
   *
   * @param newPage The page to go to.
   */
  _changePage(newPage: number): void {
    const query = new URLSearchParams(this.context?.query?.toString() ?? '');

    if (newPage !== 0) {
      query.set('page', '' + newPage);
    } else {
      query.delete('page');
    }

    this.getResults(query);
  }

  /**
   * Get the element to display the current page.
   */
  _getCurrentPageElement() {
    const currentPage = this.context?.response?.search_results_page ?? 0;

    return html`<button class="page current" aria-pressed="true">
      ${currentPage + 1}
    </button>`;
  }

  /**
   * Get pages to display before and after the current page.
   */
  _getWrappingPagesElements(): {
    prev: TemplateResult[];
    next: TemplateResult[];
  } {
    const prevPages = [];
    const nextPages = [];
    const currentPage = this.context?.response?.search_results_page ?? 0;
    const maxPage = this.context?.response?.search_results_pages ?? 0;

    for (let p = 1; p <= this.pagesToDisplay; p++) {
      if (currentPage - p >= 0) {
        prevPages.unshift(html` <button
          class="prev page"
          @click=${() => this._changePage(currentPage - p)}
        >
          ${currentPage - p + 1}
        </button>`);
      }

      if (currentPage + p < maxPage) {
        nextPages.push(html` <button
          class="next page"
          @click=${() => this._changePage(currentPage + p)}
        >
          ${currentPage + p + 1}
        </button>`);
      }
    }

    return {prev: prevPages, next: nextPages};
  }

  /**
   * Get the next, previous, first, last button elements.
   */
  _getWrappingButtonElements(): {
    prev: TemplateResult;
    next: TemplateResult;
    first: TemplateResult;
    last: TemplateResult;
  } {
    const currentPage = this.context?.response?.search_results_page ?? 0;
    const maxPages = this.context?.response?.search_results_pages ?? 0;

    const prev = html` ${this.showNextPrev
      ? html` <button
          class="prev"
          ?disabled=${currentPage === 0}
          @click=${() => this._changePage(currentPage - 1)}
        >
          ${this.prevLabel}
        </button>`
      : null}`;

    const next = html` ${this.showNextPrev
      ? html` <button
          class="next"
          ?disabled=${currentPage === maxPages - 1}
          @click=${() => this._changePage(currentPage + 1)}
        >
          ${this.nextLabel}
        </button>`
      : null}`;

    const first = html` ${this.showFirstLast && currentPage !== 0
      ? html`<button class="first" @click=${() => this._changePage(0)}>
          ${this.firstLabel}
        </button>`
      : null}`;

    const last = html` ${this.showFirstLast && currentPage !== maxPages - 1
      ? html`<button
          class="last"
          @click=${() => this._changePage(maxPages - 1)}
        >
          ${this.lastLabel}
        </button>`
      : null}`;

    return {
      next: next,
      prev: prev,
      first: first,
      last: last,
    };
  }

  override render() {
    const wrappingPages = this._getWrappingPagesElements();
    const wrappingButtons = this._getWrappingButtonElements();

    return html`
      <div class="pager">
        ${wrappingButtons.first} ${wrappingButtons.prev}
        ${wrappingPages.prev.length > 0 ? wrappingPages.prev : null}
        ${this._getCurrentPageElement()}
        ${wrappingPages.next.length > 0 ? wrappingPages.next : null}
        ${wrappingButtons.next} ${wrappingButtons.last}
      </div>
    `;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'search-simple-pager': SearchSimplePager;
  }
}
